/*
	HighscoreView.cpp
	Copyright 1996-1997 Be Do Have Software. All Rights Reserved.
*/

#include <interface/Alert.h>
#include <storage/Directory.h>
#include <storage/File.h>
#include <storage/Entry.h>
#include <string.h>
#include <errno.h>

#include "BdhApp.h"
#include "BdhEntryDialog.h"

#include "SquareView.h"
#include "HighscoreView.h"

// --------------------------------------------------------------
// HighscoreView
//		view displaying high score information
// used to sort list by score
int scoreSortFn( const void *arg1, const void *arg2 );

HighscoreView::HighscoreView( const uint16 columns, const uint16 rows,
		const BFont *font, const BPoint offset, const char* title,
		const char* scoreFileName, const char* firstline )
	: TextGridView( columns, rows, font, offset, title ),
		scoreFn(0), firstLine(0), historyList(0), listMax(rows),
		errorNum(0)
{
	scoreFn = new char[strlen(scoreFileName)+1];
	strcpy( scoreFn, scoreFileName );
	if ( firstline != 0 ) {
		firstLine = new char[strlen(firstline)+1];
		strcpy( firstLine, firstline );
		listMax--;
	}

	// fill history list
	historyList = new BList;
	BDirectory datadir( appDirpath() );
	BEntry sEn(&datadir, scoreFn);
	BFile scoreFile( &sEn, B_READ_ONLY );
	errorNum = errno;
	if ( scoreFile.IsReadable() ) {
		fillHistory( &scoreFile );
	}
}

HighscoreView::~HighscoreView()
{
	BDirectory datadir( appDirpath() );
	
	// create temp file
	char *tempFn = new char[strlen(scoreFn)+5];
	strcpy( tempFn, scoreFn );
	strcat( tempFn, ".tmp" );
	BFile scoreFile;
	datadir.CreateFile( tempFn, &scoreFile );

	// fill temp file
	char *tmp;
	for ( int i=historyList->CountItems() ; i>0 ; i-- ) {
		tmp = reinterpret_cast<char*>
			(historyList->RemoveItem(0L));
		if ( tmp == 0 ) continue;
		scoreFile.Write( tmp, strlen(tmp) );
		scoreFile.Write( "\n", 1 );
		delete [] tmp;
	}
	
	// replace score file with temp file
	BEntry en( &datadir, scoreFn );
	if ( en.InitCheck() == B_OK ) {
		en.Remove();
		en.Unset();
	}
	en.SetTo( &datadir, tempFn );
	en.Rename( scoreFn );
	
	// clean up
	delete historyList;
	delete [] firstLine;
	delete [] scoreFn;
}

void
HighscoreView::fillHistory( BFile *fp )
{
	char line[256];
	uint32 score;
	for ( int i=0 ; i<listMax ; ) {
		line[0] = '\0';
		if ( getline( fp, line, 255 ) < 0 ) {
			break;
		}
		if ( sscanf( line, "%lu", &score ) != 1 ) {
			continue;
		}
		// place copy of line onto history list
		char *tmp = new char[256];
		strcpy( tmp, line );
		historyList->AddItem(tmp);
		i++;
	}
	if ( historyList->CountItems() > 1 ) {
		historyList->SortItems( scoreSortFn );
	}
}


// -- Read until buffer fills, or newline encountered
//      Resulting string in buffer is null-terminated;
//      count of characters read is returned,
//		or negative for error
int
HighscoreView::getline( BFile *fp, char *buffer, int bufSize )
{
	// read in chunk of file, up to bufSize-1 chars
	long actual = fp->Read( (void*)buffer, (long)(bufSize-1) );
	if ( actual < 1 ) { return -1; }

	// count chars up to '\n' (if any)
	int count=0;
	for ( count=0 ; buffer[count] != '\n' &&
			count < bufSize-1 ; count++ ) { ; }
	if ( buffer[count] == '\n' ) {
		// hit a newline, adjust seek position
		fp->Seek(-actual+count+1,1);
		buffer[count]='\0';
	} else {
		buffer[bufSize-1] = '\0';
	}
	return count;
}

int
scoreSortFn( const void *arg1, const void *arg2 )
{
	// Kluge for DR9-AA:  misdeclared API
	const char *one = *(const char**)(arg1);
	const char *two = *(const char**)(arg2);
	
	uint32 score1, score2;
	if ( !one || sscanf( one, "%lu", &score1 ) != 1 ) {
		score1 = 0;
	}
	if ( !two || sscanf( two, "%lu", &score2 ) != 1 ) {
		score2 = 0;
	}
	return (score2 - score1);
}


void
HighscoreView::Draw( BRect /*rect*/ )
{
	if ( firstLine != NULL ) {
		MoveMarkTo( 0, 0 );
		DrawString( firstLine );
		StrokeLine(	BPoint(1,vertStep-1),
					BPoint(horizStep*horizCount-1,vertStep-1) );
	}
	char *tmp;
	for ( int i=0; i<listMax; i++ ) {
		if ( ! (i<historyList->CountItems()) ) { break; }
		tmp = reinterpret_cast<char*>(historyList->ItemAt(i));
		MoveMarkTo( i+((firstLine!=NULL)?1:0), 0 );
		DrawString( tmp );
	}
}


void
HighscoreView::AttachedToWindow(void)
{
	TextGridView::AttachedToWindow();
}


void
HighscoreView::alertScore( uint32 score )
{
	// alert score
	char text[32];
	sprintf( text, "Your score was %lu.", score );
	BAlert *alert = new BAlert( "Your Score", text, "Yowza!" );
	alert->Go();
	// if new high score, enter into history list
	if ( historyList->CountItems() < listMax ) {
		promptScorer( score );
	} else {
		uint32 lowestScore;
		char *tmp= reinterpret_cast<char*>(historyList->LastItem());
		if ( sscanf( tmp, "%lu", &lowestScore ) != 1 ) {
			lowestScore = 0;
		}
		if ( score >= lowestScore ) {
			promptScorer( score );
		}
	}
}


void
HighscoreView::promptScorer( uint32 score )
{
	BMessage *msg = new BMessage( SCORE_VIEW_PROMPTED );
	msg->AddInt32( "score", score );
	msg->AddString( "text", 
			"Congratulations!  You have a new high score!" );
	msg->AddString( "label", "Your name" );
	msg->AddString( "data", "Be Do Have Software" );
	BdhEntryDialog *ask = new BdhEntryDialog( BPoint(50,50),
			msg, BDH_ICON_EXCLAIM );
	ask->Go( this );
}


// Add high score to history list
void
HighscoreView::addHighScore( uint32 score, const char *scorer )
{
	if ( scorer == NULL ) {
		scorer = "Someone really SPECIAL!";
	}
	if ( historyList->CountItems() >= listMax ) {
		char *tmp = reinterpret_cast<char*>
			(historyList->LastItem());
		historyList->RemoveItem(tmp);
		delete [] tmp;
	}
	char *line = new char[256];
	sprintf( line, "%lu %s", score, scorer );
	line[255] = '\0';
	historyList->AddItem(line);
	if ( historyList->CountItems() > 1 ) {
		historyList->SortItems( scoreSortFn );
	}
	Invalidate();
}


void
HighscoreView::MessageReceived( BMessage *msg )
{
	switch (msg->what) {
	case SCORE_VIEW_PROMPTED:
	    {
		int32 score = 0;
		msg->FindInt32( "score", &score );
		const char *scorer;
		msg->FindString( "data", &scorer );
		addHighScore( score, scorer );
	    }
		break;
	default:
		TextGridView::MessageReceived(msg);
		break;
	} // switch
}
